"use strict";
/**
 * @file root-game-loading.ts
 * @copyright 2020-2024, Firaxis Games
 * @description Full screen content to show between shell and game states.
 */
///<reference path="loading-flipbook.ts" />
const WAIT_FOR_FADE_IN_TIMEOUT_DURATION = 400;
(function () {
    // ---------- start of: utilities-loading ----------
    // TODO: import this code from another module (can't for now as root-game-loading is not a module)
    const requiredLoadingComponents = [
        {
            id: UIGameLoadingProgressState.ContentIsConfigured,
            value: 5,
        },
        {
            id: UIGameLoadingProgressState.GameCoreInitializationIsStarted,
            value: 2,
        },
        {
            id: UIGameLoadingProgressState.GameIsInitialized,
            value: 2,
        },
        {
            id: UIGameLoadingProgressState.GameCoreInitializationIsDone,
            value: 4,
        },
        {
            id: UIGameLoadingProgressState.GameIsFinishedLoading,
            value: 2,
        },
        {
            id: UIGameLoadingProgressState.UIIsInitialized,
            value: 5,
        },
        {
            id: UIGameLoadingProgressState.UIIsReady,
            value: 3,
        },
    ];
    const totalRequiredLoadingComponentsValue = requiredLoadingComponents.reduce((res, { value }) => res + value, 0);
    const rootLoadingRequiredLoadingComponentIds = [
        UIGameLoadingProgressState.ContentIsConfigured,
    ];
    function getRootGameLoadingInitialLoadingRatio() {
        const initialRequiredLoadingComponentsValue = requiredLoadingComponents.reduce((res, { id, value }) => res + (rootLoadingRequiredLoadingComponentIds.includes(id) ? value : 0), 0);
        return initialRequiredLoadingComponentsValue / totalRequiredLoadingComponentsValue;
    }
    // ---------- end of: utilities-loading ----------
    const completedUIGameLoadingProgressStates = new Set();
    completedUIGameLoadingProgressStates.add(UIGameLoadingProgressState.ContentIsConfigured);
    const loadingBar = document.querySelector(".loading-curtain__loading-bar");
    const loadingBarProgress = document.createElement("div");
    loadingBarProgress.classList.add("loading-curtain__loading-bar-progress", "absolute", "bottom-0", "left-0", "h-2", "bg-secondary-1", "transition-width");
    loadingBarProgress.style.setProperty("transition-timing-function", "linear");
    loadingBarProgress.style.setProperty("width", `${getRootGameLoadingInitialLoadingRatio() * 100}%`);
    loadingBar?.appendChild(loadingBarProgress);
    // These images are necessary to show the hourglass.
    const preloadHourglass = [
        'fs://game/hourglasses01.png',
        'fs://game/hourglasses02.png',
        'fs://game/hourglasses03.png',
    ];
    // These are images we want to ensure are fully loaded before fading the screen in.
    // This should be sync'd with the CSS so that all necessary assets are loaded.
    const preloadImages = [
        'fs://game/ntf_next_turn.png',
        'fs://game/Rounded-Hex_112px.png',
        'fs://game/adv_port_bk.png',
        'fs://game/hud_unit-panel_unitframe.png'
    ];
    function Preload(images) {
        const promises = [];
        images.forEach((img) => {
            const promise = new Promise((resolve, _reject) => {
                let image = new Image();
                image.src = img;
                image.style.display = 'none';
                image.style.position = 'absolute';
                image.addEventListener('load', () => {
                    image = null;
                    resolve();
                });
                // Even if the image fails to load, continue.
                image.addEventListener('error', () => {
                    image = null;
                    resolve();
                });
            });
            promises.push(promise);
        });
        return Promise.all(promises);
    }
    function setElementContent(elementId, content) {
        const element = document.getElementById(elementId);
        if (!element) {
            console.error(`game-loading: Unable to set loading screen content '${elementId}' element not found on DOM.`);
            return;
        }
        element.setAttribute('data-l10n-id', content);
    }
    function getAgeLoadingInfo() {
        const gameConfig = Configuration.getGame();
        const playerConfig = Configuration.getPlayer(GameContext.localPlayerID);
        if (!gameConfig || !playerConfig) {
            return null;
        }
        const ageTypeName = gameConfig.startAgeName;
        const leaderTypeName = playerConfig.leaderTypeName;
        const civTypeName = playerConfig.civilizationTypeName;
        if (ageTypeName && civTypeName && leaderTypeName) {
            // Grab all valid infos.		
            let loadingInfos = GameInfo.LoadingInfo_Ages.filter(info => {
                const civTypeOverride = info.CivilizationTypeOverride;
                const leaderTypeOverride = info.LeaderTypeOverride;
                return info.AgeType == ageTypeName &&
                    (civTypeOverride == null || civTypeOverride == civTypeName) &&
                    (leaderTypeOverride == null || leaderTypeOverride == leaderTypeName);
            });
            // Sort based on overrides.
            // Sort is based on specificity descending so the first item in the array is the one we want.
            // CivilizationTypeOverride is considered more specific than LeaderTypeOverride.
            loadingInfos.sort((a, b) => {
                let a_score = ((a.CivilizationTypeOverride) ? 10 : 0) + ((a.LeaderTypeOverride) ? 1 : 0);
                let b_score = ((b.CivilizationTypeOverride) ? 10 : 0) + ((b.LeaderTypeOverride) ? 1 : 0);
                // Use less than because this is sort-descending.
                if (a_score == b_score) {
                    return 0;
                }
                else {
                    return (a_score < b_score) ? -1 : 1;
                }
            });
            // Take the first value.
            if (loadingInfos.length > 0) {
                return loadingInfos[0];
            }
        }
        return null;
    }
    function getCivLoadingInfo() {
        const gameConfig = Configuration.getGame();
        const playerConfig = Configuration.getPlayer(GameContext.localPlayerID);
        if (!gameConfig || !playerConfig) {
            return null;
        }
        const ageTypeName = gameConfig.startAgeName;
        const leaderTypeName = playerConfig.leaderTypeName;
        const civTypeName = playerConfig.civilizationTypeName;
        if (ageTypeName && civTypeName && leaderTypeName) {
            // Grab all valid infos.		
            let loadingInfos = GameInfo.LoadingInfo_Civilizations.filter(info => {
                const ageTypeOverride = info.AgeTypeOverride;
                const leaderTypeOverride = info.LeaderTypeOverride;
                return info.CivilizationType == civTypeName &&
                    (ageTypeOverride == null || ageTypeOverride == ageTypeName) &&
                    (leaderTypeOverride == null || leaderTypeOverride == leaderTypeName);
            });
            // Sort based on overrides.
            // Sort is based on specificity descending so the first item in the array is the one we want.
            // LeaderTypeOverride is considered more specific than AgeTypeOverride.
            loadingInfos.sort((a, b) => {
                let a_score = ((a.LeaderTypeOverride) ? 10 : 0) + ((a.AgeTypeOverride) ? 1 : 0);
                let b_score = ((b.LeaderTypeOverride) ? 10 : 0) + ((b.AgeTypeOverride) ? 1 : 0);
                // Use less than because this is sort-descending.
                if (a_score == b_score) {
                    return 0;
                }
                else {
                    return (a_score < b_score) ? -1 : 1;
                }
            });
            // Take the first value.
            if (loadingInfos.length > 0) {
                return loadingInfos[0];
            }
        }
        return null;
    }
    function getLeaderLoadingInfo() {
        const gameConfig = Configuration.getGame();
        const playerConfig = Configuration.getPlayer(GameContext.localPlayerID);
        if (!gameConfig || !playerConfig) {
            return null;
        }
        const ageTypeName = gameConfig.startAgeName;
        const leaderTypeName = playerConfig.leaderTypeName;
        const civTypeName = playerConfig.civilizationTypeName;
        if (ageTypeName && civTypeName && leaderTypeName) {
            // Grab all valid infos.		
            let loadingInfos = GameInfo.LoadingInfo_Leaders.filter(info => {
                const ageTypeOverride = info.AgeTypeOverride;
                const civTypeOverride = info.CivilizationTypeOverride;
                return info.LeaderType == leaderTypeName &&
                    (ageTypeOverride == null || ageTypeOverride == ageTypeName) &&
                    (civTypeOverride == null || civTypeOverride == civTypeName);
            });
            // Sort based on overrides.
            // Sort is based on specificity descending so the first item in the array is the one we want.
            // CivilizationTypeOverride is considered more specific than AgeTypeOverride.
            loadingInfos.sort((a, b) => {
                let a_score = ((a.CivilizationTypeOverride) ? 10 : 0) + ((a.AgeTypeOverride) ? 1 : 0);
                let b_score = ((b.CivilizationTypeOverride) ? 10 : 0) + ((b.AgeTypeOverride) ? 1 : 0);
                // Use less than because this is sort-descending.
                if (a_score == b_score) {
                    return 0;
                }
                else {
                    return (a_score < b_score) ? -1 : 1;
                }
            });
            // Take the first value.
            if (loadingInfos.length > 0) {
                return loadingInfos[0];
            }
        }
        return null;
    }
    function realizeInfo() {
        const playerConfig = Configuration.getPlayer(GameContext.localPlayerID);
        if (!playerConfig) {
            return;
        }
        if (!playerConfig.leaderTypeName ||
            !playerConfig.civilizationTypeName) {
            console.error("Missing necessary data for loading screen.");
            return;
        }
        const leaderName = playerConfig.leaderName ?? 'ERROR: Missing Leader Name';
        const civName = playerConfig.civilizationFullName ?? playerConfig.civilizationName ?? 'ERROR: Missing Civilizaiton Name';
        const civType = playerConfig.civilizationTypeName;
        const ageLoadingInfo = getAgeLoadingInfo();
        const civLoadingInfo = getCivLoadingInfo();
        const leaderLoadingInfo = getLeaderLoadingInfo();
        let civTrait = GameInfo.LegacyCivilizationTraits.lookup(civType);
        // Fill Data per User's Choices
        const leaderImageElement = document.querySelector(".root-game-loading__leaderPortrait");
        if (leaderImageElement) {
            const imagePath = leaderLoadingInfo?.LeaderImage;
            if (imagePath) {
                leaderImageElement.style.backgroundImage = `url(${imagePath})`;
            }
        }
        const backgroundImageElement = document.querySelector(".root-game-loading__scene-fg");
        if (backgroundImageElement) {
            const imagePath = window.innerWidth >= 1080 ? civLoadingInfo?.BackgroundImageHigh : civLoadingInfo?.BackgroundImageLow;
            if (imagePath) {
                backgroundImageElement.style.backgroundImage = `url(${imagePath})`;
            }
        }
        const { uiFontScale } = Configuration.getUser();
        // for medium scale and up, enable the leader/civ icons since you can't see the backdrop
        const smallIconsBox = document.querySelector(".root-game-loading__small-screen-area");
        if (smallIconsBox) {
            if (uiFontScale >= 1 /* FontScale.Small */) {
                smallIconsBox.classList.remove("hidden");
            }
        }
        // For medium & large
        if (uiFontScale >= 2 /* FontScale.Medium */) {
            // scoot the leader/civ/small uniques div down a little
            if (smallIconsBox) {
                smallIconsBox.classList.remove("-bottom-3");
                smallIconsBox.classList.add("-bottom-8");
            }
            // For large
            if (uiFontScale >= 3 /* FontScale.Large */) {
                // move the click-to-start button to the right edge of the screen
                const clickToStart = document.getElementById("start_game");
                if (clickToStart) {
                    clickToStart.classList.add("root-game-loading-icon-large");
                }
                // hide the main unique units/buildings area
                const uniqueIconsArea = document.querySelector(".root-game-loading__unique-horizontal-flex-box");
                if (uniqueIconsArea) {
                    uniqueIconsArea.classList.add("root-game-loading-uniques-large");
                }
                // show the auxiliary unique units/buildings area
                const smallScreenUniques = document.querySelector(".root-game-loading__small-screen-uniques");
                if (smallScreenUniques) {
                    smallScreenUniques.classList.add("root-game-loading-small-screen-uniques-large");
                }
            }
            else { // Medium, just move click-to-start
                // move the click-to-start button to the right edge of the screen
                const clickToStart = document.getElementById("start_game");
                if (clickToStart) {
                    clickToStart.classList.add("root-game-loading-icon-medium");
                }
            }
        }
        // Update age-specific info
        setElementContent('root-game-loading__intro', civLoadingInfo?.Subtitle ?? ageLoadingInfo?.AgeIntroText ?? 'LOC_LOADING_GENERIC_INTRO_TEXT');
		const leaderNameText = leaderLoadingInfo?.LeaderNameTextOverride ?? leaderName;
        let leaderIntroText =  `[STYLE:text-secondary font-title uppercase tracking-100 font-bold]{${leaderNameText}}[/S] `;
        if (leaderLoadingInfo?.LeaderText) {
            leaderIntroText +=   `{${leaderLoadingInfo.LeaderText}}`;
        }
        setElementContent("root-game-loading__leader-info-field", leaderIntroText);
        const civNameText = civLoadingInfo?.CivilizaitonNameTextOverride ?? civName;
        let civilizationIntroText = `[STYLE:text-secondary font-title uppercase tracking-100 font-bold]{${civNameText}}[/S] `;
        if (civLoadingInfo?.CivilizationText) {
            civilizationIntroText +=   `{${civLoadingInfo.CivilizationText}}`;
        }
        setElementContent("root-game-loading__civilization-info-field", civilizationIntroText);
        const audioTag = ageLoadingInfo?.AudioOverride ?? leaderLoadingInfo?.AudioOverride ?? civLoadingInfo?.Audio;
        if (audioTag) {
            UI.sendAudioEvent(audioTag);
        }
        const smallLeaderIcon = document.querySelector(".root-game-loading-small-leader-icon");
        if (smallLeaderIcon && leaderLoadingInfo) {
            smallLeaderIcon.style.backgroundImage = `${UI.getIconCSS(leaderLoadingInfo.LeaderType, "LEADER")}`;
        }
        const smallCivIcon = document.querySelector(".root-game-loading-small-civ-icon");
        if (smallCivIcon && civLoadingInfo) {
            const civIconURL = "url('fs://game/civ_sym_" + civLoadingInfo.CivilizationType.slice(13).toLowerCase() + ".png')";
            smallCivIcon.style.backgroundImage = `${civIconURL}`;
        }
        // Find all the exclusive things for this civilization
        const leaderTraitContainer = document.querySelector(".root-game-loading_FH-leader-trait-flex-box");
		const uniqueTraitContainer = document.querySelector(".root-game-loading_FH-civ-trait-flex-box");
		const uniqueUnitsContainer = document.querySelector(".root-game-loading_FH-unique-units-flex-box");
        const smallScreenUniques = document.querySelector(".root-game-loading__small-screen-uniques");
		setElementContent("root-game-loading__unique-units-field", 'LOC_LOADING_UNIQUE_UNITS_TEXT');
        let uniqueCount = 0;
		// abilities
		const leaderBonusItems = Database.query('config', 'select * from LeaderItems order by SortIndex')?.filter(item => item.LeaderType == leaderLoadingInfo.LeaderType);
        const leaderBonusTrait = leaderBonusItems?.find(item => item.Kind == "KIND_TRAIT");
		const civBonusItems = Database.query('config', 'select * from CivilizationItems order by SortIndex')?.filter(item => item.CivilizationType == civLoadingInfo.CivilizationType);
        const civBonusTrait = civBonusItems?.find(item => item.Kind == "KIND_TRAIT");
		const uiAutoScaleValue = Configuration.getUser().uiAutoScale;
		if (uiFontScale <= 1 && uiAutoScaleValue==false) {
			setElementContent("root-game-loading__civ-trait-field", 'LOC_LOADING_CIV_TRAIT_TEXT');
			setElementContent("root-game-loading__leader-trait-field", 'LOC_LOADING_LEADER_TRAIT_TEXT');
			if (leaderTraitContainer && smallScreenUniques) {
				let leaderTraitDescription =  leaderBonusTrait?.Description ? leaderBonusTrait?.Description : "";
				setElementContent("root-game-loading__leader-trait-text", leaderTraitDescription);
			}
			if (uniqueTraitContainer && smallScreenUniques) {
				let civTraitDescription =  civBonusTrait?.Description ? civBonusTrait?.Description : "";
				setElementContent("root-game-loading__civ-trait-text", civTraitDescription);
			}
		}
		const uniqueInfrastructureContainer = document.querySelector(".root-game-loading_unique-infrastructure-flex-box");
		setElementContent("root-game-loading__unique-infrastructure-field", 'LOC_LOADING_UNIQUE_INFRASTRUCTURE_TEXT');
		const civUniqueItems = civBonusItems?.filter(item => item.Kind == 'KIND_BUILDING' || item.Kind == 'KIND_IMPROVEMENT' || item.Kind == 'KIND_UNIT' || item.Kind == "KIND_QUARTER" || item.Kind == "KIND_ROUTE");
		civUniqueItems?.forEach(uniqueItem => {
            let iconName = "";
            let typeName = "";
            switch (uniqueItem.Kind) {
                case 'KIND_UNIT':
					const unitIcon = UI.getIconCSS(uniqueItem.Type, "UNIT_FLAG");
					addUniqueCallout(uniqueUnitsContainer, smallScreenUniques, uniqueItem?.Name, uniqueItem?.Description ? uniqueItem?.Description : "", unitIcon);
					uniqueCount++;
                    break;
                case 'KIND_IMPROVEMENT':
					const improvementIcon = UI.getIconCSS(uniqueItem.Type, "IMPROVEMENT");
                    addUniqueCallout(uniqueInfrastructureContainer, smallScreenUniques, uniqueItem?.Name, uniqueItem?.Description ? uniqueItem?.Description : "", improvementIcon);
                    uniqueCount++;
                    break;
                case 'KIND_BUILDING':
					const buildingIcon = UI.getIconCSS(uniqueItem.Type, "BUILDING");
                    addUniqueCallout(uniqueInfrastructureContainer, smallScreenUniques, uniqueItem?.Name, uniqueItem?.Description ? uniqueItem?.Description : "", buildingIcon);
                    uniqueCount++;
                    break;
                case 'KIND_QUARTER':
					const quarterIcon = UI.getIconCSS("CITY_UNIQUE_QUARTER");
                    addUniqueCallout(uniqueInfrastructureContainer, smallScreenUniques, uniqueItem?.Name, uniqueItem?.Description ? uniqueItem?.Description : "", quarterIcon);
                    uniqueCount++;
                    break;
				case 'KIND_ROUTE':
					const routeIcon = UI.getIconCSS("BUILDING_RAIL_STATION");
                    addUniqueCallout(uniqueInfrastructureContainer, smallScreenUniques, uniqueItem?.Name, uniqueItem?.Description ? uniqueItem?.Description : "", routeIcon);
                    uniqueCount++;
                    break;
                default:
					break;
            }
		});
    }
    function addUniqueCallout(parent, smallParent, name, description, iconURL) {
        const { uiFontScale } = Configuration.getUser();
        const container = document.createElement("fxs-hslot");
        container.classList.add("root-game-loading__unique-units-flex-box", "flex-auto");
        container.classList.remove("flex-wrap");
        if (uiFontScale > 1 /* FontScale.Small */) {
            if (uiFontScale == 4 /* FontScale.XLarge */) {
                container.classList.add("root-game-loading__four-column-unique");
            }
            else {
                container.classList.add("root-game-loading__two-column-unique");
            }
        }
        parent.appendChild(container);
        container.innerHTML = `
			<div class="root-game-loading__unique-individual-unit-portrait-flexbox flex relative">
				<div class="root-game-loading__unique-individual-unit-portrait-outer-hex absolute inset-0 flex bg-contain bg-no-repeat justify-center">
					<div class="root-game-loading__unique-individual-unit-portrait-masked absolute bg-cover bg-center bg-bottom inset-y-3" style="background-image: ${iconURL}"></div>
				</div>
			</div>
			<div class="root-game-loading__unique-individual-unit-description-flexbox flex flex-auto flex-col">
				<div class="root-game-loading__unique-individual-unit-title-text font-title-base" data-l10n-id="${name}">
				</div>
				<div class="root-game-loading__unique-individual-unit-description-text font-body-sm text-secondary" data-l10n-id="${description}">
				</div>
			</div>
		`;
        const smallContainer = document.createElement("div");
        smallParent.appendChild(smallContainer);
        smallContainer.innerHTML = `
			<div class="root-game-loading__unique-individual-unit-portrait-flexbox relative">
				<div class="root-game-loading__unique-individual-unit-portrait-outer-hex absolute inset-0 flex bg-contain bg-no-repeat justify-center">
					<div class="root-game-loading__unique-individual-unit-portrait-masked absolute bg-cover bg-center bg-bottom inset-y-3" style="background-image: ${iconURL}"></div>
				</div>
			</div>
		`;
    }
    function showLoadingAnimation() {
        const animButtonContainer = document.getElementById('loading-screen__animation-container');
        if (animButtonContainer) {
            const flipbook = document.createElement("flip-book");
            const flipbookDefinition = {
                fps: 30,
                atlas: [['fs://game/hourglasses01.png', 128, 128, 512],
                    ['fs://game/hourglasses02.png', 128, 128, 512],
                    ['fs://game/hourglasses03.png', 128, 128, 1024, 13]]
            };
            flipbook.setAttribute("data-flipbook-definition", JSON.stringify(flipbookDefinition));
            animButtonContainer.appendChild(flipbook);
            animButtonContainer.classList.remove('hidden');
        }
        else {
            console.error("game-loading: Unable to find container for loading gif.");
        }
    }
    function hideLoadingAnimation() {
        const animButtonContainer = document.getElementById('loading-screen__animation-container');
        if (animButtonContainer) {
            animButtonContainer.remove();
        }
        else {
            console.error("game-loading: Unable to find container for loading gif.");
        }
    }
    function createStartButton(isFirstGame) {
        const startContainer = document.getElementById('start_game');
        if (!startContainer) {
            console.error("game-loading: Unable to find container for continue button.");
            return;
        }
        const fragment = document.createDocumentFragment();
        const continueButtonParent = document.createElement("fxs-activatable");
        continueButtonParent.classList.add("root-game-loading__continue-button");
        continueButtonParent.addEventListener('action-activate', () => {
            continueButtonParent.classList.toggle('disabled', true);
            startTextElement.setAttribute('data-l10n-id', 'LOC_LOADING_WAITING_TO_START');
            loadingNavHelpIcon.classList.add('hidden');
            UI.notifyUIReady();
            if (Configuration.getXR()) {
                XR.Atlas.invokeEvent(EViewID.MultiquadFrame, "begin-game", "");
            }
        });
        continueButtonParent.setAttribute('data-audio-group-ref', 'main-menu-audio');
        const activateSoundTag = isFirstGame ? 'data-audio-begin-game' : 'data-audio-continue-game';
        const pressSoundTag = isFirstGame ? 'data-audio-begin-game-press' : 'data-audio-continue-game-press';
        continueButtonParent.setAttribute('data-audio-activate-ref', activateSoundTag);
        continueButtonParent.setAttribute('data-audio-press-ref', pressSoundTag);
        const continueButtonOutline = document.createElement("div");
        continueButtonOutline.classList.add("root-game-loading__continue-button-bg", "continue-button--outline", "inset-4");
        continueButtonParent.appendChild(continueButtonOutline);
        const continueButtonHighlight = document.createElement("div");
        continueButtonHighlight.classList.add("root-game-loading__continue-button-bg", "continue-button--highlight");
        continueButtonParent.appendChild(continueButtonHighlight);
        const continueButtonIcon = document.createElement("div");
        continueButtonIcon.classList.add("root-game-loading__continue-button-icon");
        continueButtonParent.appendChild(continueButtonIcon);
        fragment.appendChild(continueButtonParent);
        const loadingContainerElement = document.createElement("div");
        loadingContainerElement.classList.add("root-game-loading__start-game-text-container", '-ml-2');
        loadingContainerElement.classList.add("trigger-nav-help");
        const loadingNavHelpIcon = document.createElement("fxs-nav-help");
        loadingNavHelpIcon.setAttribute('action-key', 'inline-accept');
        loadingContainerElement.appendChild(loadingNavHelpIcon);
        const startTextElement = document.createElement("div");
        startTextElement.classList.add("root-game-loading__loading-gif-text", "font-bold");
        const startButtonText = isFirstGame ? 'LOC_LOADING_BEGIN_GAME' : 'LOC_LOADING_CONTINUE_GAME';
        startTextElement.setAttribute('data-l10n-id', startButtonText);
        loadingContainerElement.appendChild(startTextElement);
        fragment.appendChild(loadingContainerElement);
        startContainer.appendChild(fragment);
        delayByFrame(() => {
            startContainer.style.opacity = "1";
        }, 16);
    }
    function updateLoadingBarWidthRatio(numerator, denominator) {
        const ratio = numerator / denominator;
        loadingBarProgress?.classList.toggle("transition-width", ratio != 1);
        loadingBarProgress?.style.setProperty("width", `${ratio * 100}%`);
    }
    function onUIGameLoadingProgressChanged({ UIGameLoadingProgressState }) {
        handleUIGameLoadingProgressChanged(UIGameLoadingProgressState);
    }
    function handleUIGameLoadingProgressChanged(UIGameLoadingProgressState) {
        const timeoutDuration = completedUIGameLoadingProgressStates.size == 1 ? WAIT_FOR_FADE_IN_TIMEOUT_DURATION : 0; // we want to wait for the fade in before progressing the loading bar
        completedUIGameLoadingProgressStates.add(UIGameLoadingProgressState);
        const states = [...completedUIGameLoadingProgressStates];
        const totalCurrentLoadingComponentsValue = requiredLoadingComponents.reduce((res, { id, value }) => res + (states.includes(id) ? value : 0), 0);
        setTimeout(() => {
            updateLoadingBarWidthRatio(totalCurrentLoadingComponentsValue, totalRequiredLoadingComponentsValue);
        }, timeoutDuration);
    }
    // If the game has already started, just skip all of this since the loading curtain will be pulled this frame.
    if (UI.getGameLoadingState() != UIGameLoadingState.GameStarted) {
        engine.whenReady.then(() => {
            engine.on('UIGameLoadingProgressChanged', onUIGameLoadingProgressChanged);
        });
        const preloadedHourglass = Preload(preloadHourglass);
        preloadedHourglass.then(() => {
            showLoadingAnimation();
            const el = document.getElementById('loading-root');
            if (el) {
                el.classList.add('fade-in');
                el.classList.remove('hidden');
            }
            // Wait until the next frame before signalling the curtain is ready to ensure everything is smooth.
            requestAnimationFrame(() => {
                UI.notifyLoadingCurtainReady();
                const preloaded = Preload(preloadImages);
                preloaded.then(() => {
                    Loading.runWhenInitialized(() => {
                        realizeInfo();
                        const el = document.getElementById('loading-content-root');
                        if (el) {
                            el.classList.add('fade-in');
                            el.classList.remove('hidden');
                        }
                        else {
                            console.error("game-loading: Can't apply hide/fade styles to loading's root");
                        }
                    });
                    Loading.runWhenLoaded(() => {
                        // add all remaining game loading states to cover up for the case of reloading the root-game-loading (when everything has already been loaded)
                        completedUIGameLoadingProgressStates.add(UIGameLoadingProgressState.GameCoreInitializationIsDone);
                        completedUIGameLoadingProgressStates.add(UIGameLoadingProgressState.GameCoreInitializationIsStarted);
                        completedUIGameLoadingProgressStates.add(UIGameLoadingProgressState.GameIsFinishedLoading);
                        completedUIGameLoadingProgressStates.add(UIGameLoadingProgressState.GameIsInitialized);
                        completedUIGameLoadingProgressStates.add(UIGameLoadingProgressState.UIIsInitialized);
                        handleUIGameLoadingProgressChanged(UIGameLoadingProgressState.UIIsReady);
                        UI.lockCursor(false); // Resume normal cursor functionality
                        hideLoadingAnimation();
                        if (UI.getGameLoadingState() == UIGameLoadingState.WaitingForUIReady) {
                            const gameConfig = Configuration.getGame();
                            const isEraTransition = gameConfig.previousAgeCount > 0;
                            const isFirstGame = !gameConfig.isSavedGame || !isEraTransition;
                            createStartButton(isFirstGame);
                            UI.sendAudioEvent('main-menu-load-ready');
                        }
                    });
                });
            });
        });
    }
}()); // Immediately execute this

//# sourceMappingURL=file:///base-standard/ui/loading/root-game-loading.js.map
